
!------------------------------------------------------------------------!
!  The Community Multiscale Air Quality (CMAQ) system software is in     !
!  continuous development by various groups and is based on information  !
!  from these groups: Federal Government employees, contractors working  !
!  within a United States Government contract, and non-Federal sources   !
!  including research institutions.  These groups give the Government    !
!  permission to use, prepare derivative works of, and distribute copies !
!  of their work in the CMAQ system to the public and to permit others   !
!  to do so.  The United States Environmental Protection Agency          !
!  therefore grants similar permission to use the CMAQ system software,  !
!  but users are requested to provide copies of derivative works or      !
!  products designed to operate in the CMAQ system to the United States  !
!  Government without restrictions as to use by others.  Software        !
!  that is used with the CMAQ system but distributed under the GNU       !
!  General Public License or the GNU Lesser General Public License is    !
!  subject to their copyright restrictions.                              !
!------------------------------------------------------------------------!


      SUBROUTINE HRG2( DTC )
C**********************************************************************
C
C  FUNCTION: To solve for the concentration of HO, HO2, HONO, and
C            HNO4 alebraically.
C
C  PRECONDITIONS: For the CRACMM1_AQ mechanism
C
C  KEY SUBROUTINES/FUNCTIONS CALLED: None
C
C  REVISION HISTORY: Created by EBI solver program, Mar 12, 2022
C
C   18 Jul 14 B.Hutzell: revised to use real(8) variables
C**********************************************************************
      USE HRDATA

      IMPLICIT NONE


C..INCLUDES: None


C..ARGUMENTS:
      REAL( 8 ), INTENT( IN ) :: DTC                      ! Time step

C..PARAMETERS: None


C..EXTERNAL FUNCTIONS: NONE


C..SAVED LOCAL VARIABLES:
!     CHARACTER( 16 ), SAVE :: PNAME = 'HRG2'             ! Program name


C..SCRATCH LOCAL VARIABLES:
      REAL( 8 ) ::    O1D_S                 ! sum of O1D loss frequencies
      REAL( 8 ) ::    OH_S                  ! stoich coeff for OH from O1D+H2O or H2
      REAL( 8 ) ::    HO2_S                 ! stoich coeff for HO2 from O1D+H2 rxn
      REAL( 8 ) ::    XOH_S                 ! stoich coeff for OH & HONO from NO2EX loss rxn
      REAL( 8 ) ::    R4_19                 ! production of OH from HONO
      REAL( 8 ) ::    R19_4                 ! production of HONO from OH
      REAL( 8 ) ::    R4_5                  ! production of OH from HO2
      REAL( 8 ) ::    R5_4                  ! production of HO2 from OH
      REAL( 8 ) ::    R5_21                 ! production of HO2 from HNO4
      REAL( 8 ) ::    R21_5                 ! production of HNO4 from HO2
      REAL( 8 ) ::    P4, P5, P19, P21      ! species production form other paths
      REAL( 8 ) ::    L4, L5, L19, L21      ! species loss terms

      REAL( 8 ) ::    A, B, C               ! coeffs of quadratic eq. for HO2
      REAL( 8 ) ::    Q                     ! intermediate term

      REAL( 8 ) ::    T1, T2, T3            ! intermediate terms

      REAL( 8 ) ::    L21_INV               ! reciprocal of HNO4 loss term

C**********************************************************************


c..stoichiometric coefficient for production of HOX from O3 via O1D
      O1D_S =                 RKI(    44 )                        ! O1D=O3P
     &      +                 RKI(    45 )                        ! O1D=O3P
     &      +                 RKI(    46 )                        ! O1D=0.2000D+01*HO

      OH_S =    2.0000D+00 * RKI(    46 )                        ! O1D=0.2000D+01*HO

      OH_S  = OH_S / O1D_S


c..Production of HO from HO2 (r4,5 terms )
      R4_5 =                 RKI(    39 ) * YC ( O3          )   ! HO2+O3=HO
     &     +                 RKI(    54 ) * YC ( NO          )   ! HO2+NO=HO+NO2
     &     +    7.0000D-01 * RKI(    63 ) * YC ( NO3         )   ! HO2+NO3=0.7000D+...
     &     +                 RKI(    93 ) * YC ( INO2        )   ! HO2+INO2=HO+IPN
     &     +    4.4000D-01 * RKI(   245 ) * YC ( ACO3        )   ! HO2+ACO3=0.4400D+...
     &     +    4.4000D-01 * RKI(   246 ) * YC ( RCO3        )   ! HO2+RCO3=0.4400D+...
     &     +    1.5000D-01 * RKI(   247 ) * YC ( ACTP        )   ! HO2+ACTP=0.1500D+...
     &     +    4.0000D-01 * RKI(   404 ) * YC ( FURANO2     )   ! HO2+FURANO2=...

      R4_5  = R4_5 * DTC

c..Production of HO from HONO (r4,19 terms )
      R4_19 =                 RKI(     7 )                        ! HONO=HO+NO

      R4_19 = R4_19 * DTC

c..Remaining HO production
      P4 =    2.0000D+00 * RXRAT(     3 )      ! H2O2=0.2000D+01*HO
     &   +                 RXRAT(     8 )      ! HNO3=HO+NO2
     &   +    2.0000D-01 * RXRAT(     9 )      ! HNO4=0.2000D+00*HO+0.8000D+...
     &   +    3.4000D-01 * RXRAT(    22 )      ! MACR=0.3400D+00*HO+0.6600D+...
     &   +                 RXRAT(    31 )      ! OP1=HO+HO2+HCHO
     &   +                 RXRAT(    32 )      ! OP2=HO+HO2+ALD
     &   +                 RXRAT(    33 )      ! OPB=HO+HO2+ALD
     &   +                 RXRAT(    34 )      ! PAA=HO+MO2
     &   +   OH_S  *       RXRAT(     2 )      ! O1D=0.2000D+01*HO
     &   +    2.5000D-01 * RXRAT(    87 )      ! ISO+O3=0.2500D+00*HO+...
     &   +    8.0000D-02 * RXRAT(   142 )      ! ETE+O3=0.8000D-01*HO+...
     &   +    2.2000D-01 * RXRAT(   143 )      ! OLT+O3=0.2200D+00*HO+...
     &   +    4.6000D-01 * RXRAT(   144 )      ! OLI+O3=0.4600D+00*HO+...
     &   +    9.0000D-01 * RXRAT(   145 )      ! API+O3=0.9000D+00*HO+...
     &   +    8.4000D-01 * RXRAT(   146 )      ! LIM+O3=0.8400D+00*HO+...
     &   +    4.0000D-02 * RXRAT(   147 )      ! LIMAL+O3=0.4000D-01*HO+...
     &   +    1.9000D-01 * RXRAT(   149 )      ! MACR+O3=0.1900D+00*HO+...
     &   +    1.6000D-01 * RXRAT(   150 )      ! MVK+O3=0.1600D+00*HO+...
     &   +    1.0000D-01 * RXRAT(   151 )      ! UALD+O3=0.1000D+00*HO+...
     &   +    5.0000D-02 * RXRAT(   152 )      ! DCB1+O3=0.5000D-01*HO+HO2+...
     &   +    5.0000D-02 * RXRAT(   153 )      ! DCB2+O3=0.5000D-01*HO+HO2+...
     &   +    5.0000D-02 * RXRAT(   154 )      ! DCB3+O3=0.5000D-01*HO+HO2+...
     &   +    8.2000D-01 * RXRAT(   193 )      ! APIP2+NO=0.8200D+00*HO+...
     &   +    8.2000D-01 * RXRAT(   195 )      ! APINP2+NO=0.8200D+00*HO+...
     &   +    7.7000D-01 * RXRAT(   197 )      ! LIMP2+NO=0.7700D+00*HO+...
     &   +    7.7000D-01 * RXRAT(   199 )      ! LIMNP2+NO=0.7700D+00*HO+...
     &   +    5.0000D-01 * RXRAT(   312 )      ! APIP2+ACO3=0.5000D+00*HO+...
     &   +    5.0000D-01 * RXRAT(   316 )      ! LIMP2+ACO3=0.5000D+00*HO+...
     &   +    4.8000D-01 * RXRAT(   375 )      ! APIP2+APIP1=0.4800D+00*HO+...
     &   +    4.8000D-01 * RXRAT(   376 )      ! APIP2+LIMP1=0.4800D+00*HO+...
     &   +    4.8000D-01 * RXRAT(   377 )      ! APIP2+ISOP=0.4800D+00*HO+...
     &   +    4.8000D-01 * RXRAT(   378 )      ! LIMP2+APIP1=0.4800D+00*HO+...
     &   +    4.8000D-01 * RXRAT(   379 )      ! LIMP2+LIMP1=0.4800D+00*HO+...
     &   +    4.8000D-01 * RXRAT(   380 )      ! LIMP2+ISOP=0.4800D+00*HO+...
     &   +    2.8000D-01 * RXRAT(   391 )      ! ACRO+O3=0.2800D+00*HO+...
     &   +    2.0400D-01 * RXRAT(   393 )      ! ACRO=0.2040D+00*HO+0.4770D+...
     &   +    8.0000D-02 * RXRAT(   400 )      ! BDE13+O3=0.8000D-01*HO+...
     &   +    2.0000D-02 * RXRAT(   406 )      ! FURAN+O3=0.2000D-01*HO+ALD

      P4 = YC0( HO ) + P4 * DTC

c..Production of HO2 from OH ( r5,4 terms )
      R5_4 =                 RKI(    38 ) * YC ( O3          )   ! HO+O3=HO2
     &     +                 RKI(    47 )                        ! HO=HO2
     &     +                 RKI(    51 ) * YC ( H2O2        )   ! HO+H2O2=HO2
     &     +                 RKI(    62 ) * YC ( NO3         )   ! HO+NO3=HO2+NO2
     &     +                 RKI(    73 ) * YC ( SO2         )   ! HO+SO2=HO2+SULF+...
     &     +                 RKI(    74 ) * YC ( CO          )   ! HO+CO=HO2
     &     +    3.5000D-01 * RKI(    82 ) * YC ( ACE         )   ! HO+ACE=0.3500D+...
     &     +    5.3000D-01 * RKI(    83 ) * YC ( BEN         )   ! HO+BEN=0.5300D+...
     &     +    1.8000D-01 * RKI(    84 ) * YC ( TOL         )   ! HO+TOL=0.1800D+...
     &     +    1.7000D-01 * RKI(    85 ) * YC ( XYM         )   ! HO+XYM=0.1700D+...
     &     +    1.8000D-01 * RKI(    86 ) * YC ( XYE         )   ! HO+XYE=0.1800D+...
     &     +                 RKI(   107 ) * YC ( HCHO        )   ! HO+HCHO=HO2+CO
     &     +                 RKI(   113 ) * YC ( HKET        )   ! HO+HKET=HO2+MGLY
     &     +                 RKI(   117 ) * YC ( GLY         )   ! HO+GLY=HO2+...
     &     +    5.2000D-01 * RKI(   119 ) * YC ( DCB1        )   ! HO+DCB1=0.5200D+...
     &     +    5.2000D-01 * RKI(   120 ) * YC ( DCB2        )   ! HO+DCB2=0.5200D+...
     &     +    5.6000D-01 * RKI(   121 ) * YC ( DCB3        )   ! HO+DCB3=0.5600D+...
     &     +    6.1900D-01 * RKI(   123 ) * YC ( PHEN        )   ! HO+PHEN=0.6190D+...
     &     +    5.8400D-01 * RKI(   124 ) * YC ( CSL         )   ! HO+CSL=0.5840D+...
     &     +                 RKI(   126 ) * YC ( MOH         )   ! HO+MOH=HO2+HCHO
     &     +                 RKI(   127 ) * YC ( EOH         )   ! HO+EOH=HO2+ACD
     &     +                 RKI(   128 ) * YC ( ROH         )   ! HO+ROH=HO2+...
     &     +                 RKI(   129 ) * YC ( ETEG        )   ! HO+ETEG=HO2+ALD
     &     +                 RKI(   134 ) * YC ( ORA1        )   ! HO+ORA1=HO2
     &     +    3.3300D-01 * RKI(   394 ) * YC ( BDE13       )   ! HO+BDE13=0.3330D+...
     &     +    4.9000D-01 * RKI(   402 ) * YC ( FURAN       )   ! HO+FURAN=0.4900D+...
     &     +    6.6000D-01 * RKI(   405 ) * YC ( FURANONE    )   ! HO+FURANONE=...
     &     +                 RKI(   408 ) * YC ( PROG        )   ! HO+PROG=HO2+...
     &     +    1.6000D-01 * RKI(   484 ) * YC ( VROCP6ARO   )   ! HO+VROCP6ARO=...
     &     +    1.6000D-01 * RKI(   490 ) * YC ( VROCP5ARO   )   ! HO+VROCP5ARO=...
     &     +    1.6000D-01 * RKI(   496 ) * YC ( NAPH        )   ! HO+NAPH=0.1600D+...

      R5_4  = R5_4 * DTC

c..Production of HO2 from HNO4 (r5,21 term )
      R5_21 =    8.0000D-01 * RKI(     9 )                        ! HNO4=0.8000D+...
     &      +                 RKI(    71 )                        ! HNO4=HO2+NO2

      R5_21 = R5_21 * DTC

c..Remaining HO2 production terms
      P5 =    2.0000D+00 * RXRAT(    11 )      ! HCHO=0.2000D+01*HO2+CO
     &   +                 RXRAT(    12 )      ! ACD=HO2+MO2+CO
     &   +                 RXRAT(    13 )      ! ALD=HO2+ETHP+CO
     &   +    1.2200D+00 * RXRAT(    16 )      ! UALD=0.1220D+01*HO2+...
     &   +                 RXRAT(    17 )      ! PINAL=HO2+HC10P+CO
     &   +                 RXRAT(    18 )      ! LIMAL=HO2+HC10P+CO
     &   +                 RXRAT(    21 )      ! HKET=HO2+ACO3+HCHO
     &   +    6.6000D-01 * RXRAT(    22 )      ! MACR=0.6600D+00*HO2+...
     &   +    2.0000D+00 * RXRAT(    26 )      ! GLY=0.2000D+01*HO2+0.2000D+...
     &   +                 RXRAT(    27 )      ! MGLY=HO2+ACO3+CO
     &   +    1.5000D+00 * RXRAT(    28 )      ! DCB1=0.1500D+01*HO2+...
     &   +    1.5000D+00 * RXRAT(    29 )      ! DCB2=0.1500D+01*HO2+...
     &   +                 RXRAT(    30 )      ! BALD=HO2+CHO+CO
     &   +                 RXRAT(    31 )      ! OP1=HO2+HO+HCHO
     &   +                 RXRAT(    32 )      ! OP2=HO2+HO+ALD
     &   +                 RXRAT(    33 )      ! OPB=HO2+HO+ALD
     &   +                 RXRAT(    35 )      ! ONIT=HO2+NO2+0.2000D+...
     &   +    2.5000D-01 * RXRAT(    87 )      ! ISO+O3=0.2500D+00*HO2+...
     &   +    8.8000D-01 * RXRAT(    91 )      ! ISOP+NO=0.8800D+00*HO2+...
     &   +                 RXRAT(    94 )      ! INO2+NO=HO2+0.5000D+...
     &   +                 RXRAT(    97 )      ! IHN+NO=HO2+0.9000D+00*NO2+...
     &   +    1.5000D-01 * RXRAT(   142 )      ! ETE+O3=0.1500D+00*HO2+...
     &   +    3.2000D-01 * RXRAT(   143 )      ! OLT+O3=0.3200D+00*HO2+...
     &   +    7.0000D-02 * RXRAT(   144 )      ! OLI+O3=0.7000D-01*HO2+...
     &   +    4.0000D-02 * RXRAT(   147 )      ! LIMAL+O3=0.4000D-01*HO2+...
     &   +    1.4000D-01 * RXRAT(   149 )      ! MACR+O3=0.1400D+00*HO2+...
     &   +    1.1000D-01 * RXRAT(   150 )      ! MVK+O3=0.1100D+00*HO2+...
     &   +    7.2000D-02 * RXRAT(   151 )      ! UALD+O3=0.7200D-01*HO2+...
     &   +                 RXRAT(   152 )      ! DCB1+O3=HO2+0.5000D-01*HO+...
     &   +                 RXRAT(   153 )      ! DCB2+O3=HO2+0.5000D-01*HO+...
     &   +                 RXRAT(   154 )      ! DCB3+O3=HO2+0.5000D-01*HO+...
     &   +                 RXRAT(   162 )      ! HCHO+NO3=HO2+CO+HNO3
     &   +                 RXRAT(   166 )      ! UALD+NO3=HO2+XO2+0.6680D+...
     &   +                 RXRAT(   167 )      ! GLY+NO3=HO2+0.2000D+01*CO+HNO3
     &   +                 RXRAT(   181 )      ! MO2+NO=HO2+NO2+HCHO
     &   +                 RXRAT(   182 )      ! ETHP+NO=HO2+NO2+ACD
     &   +    6.6000D-01 * RXRAT(   183 )      ! HC3P+NO=0.6600D+00*HO2+...
     &   +    2.0000D-01 * RXRAT(   184 )      ! HC5P+NO=0.2000D+00*HO2+...
     &   +                 RXRAT(   185 )      ! ETEP+NO=HO2+NO2+0.1600D+...
     &   +    7.8000D-01 * RXRAT(   186 )      ! OLTP+NO=0.7800D+00*HO2+...
     &   +    8.3000D-01 * RXRAT(   187 )      ! OLIP+NO=0.8300D+00*HO2+...
     &   +    9.9800D-01 * RXRAT(   188 )      ! BENP+NO=0.9980D+00*HO2+...
     &   +    9.9800D-01 * RXRAT(   189 )      ! TOLP+NO=0.9980D+00*HO2+...
     &   +    9.9800D-01 * RXRAT(   190 )      ! XYMP+NO=0.9980D+00*HO2+...
     &   +    9.9800D-01 * RXRAT(   191 )      ! XYEP+NO=0.9980D+00*HO2+...
     &   +    8.2000D-01 * RXRAT(   192 )      ! APIP1+NO=0.8200D+00*HO2+...
     &   +    7.7000D-01 * RXRAT(   196 )      ! LIMP1+NO=0.7700D+00*HO2+...
     &   +    9.5000D-01 * RXRAT(   200 )      ! PINALP+NO=0.9500D+00*HO2+...
     &   +    9.4000D-01 * RXRAT(   201 )      ! LIMALP+NO=0.9400D+00*HO2+...
     &   +    6.7000D-01 * RXRAT(   205 )      ! MEKP+NO=0.6700D+00*HO2+NO2+...
     &   +    7.7000D-01 * RXRAT(   206 )      ! KETP+NO=0.7700D+00*HO2+...
     &   +    5.0000D-01 * RXRAT(   208 )      ! MCP+NO=0.5000D+00*HO2+NO2+...
     &   +    3.0000D-01 * RXRAT(   209 )      ! MVKP+NO=0.3000D+00*HO2+...
     &   +                 RXRAT(   210 )      ! UALP+NO=HO2+NO2+0.6100D+...
     &   +                 RXRAT(   213 )      ! ADDC+NO=HO2+NO2+0.3200D+...
     &   +                 RXRAT(   215 )      ! ORAP+NO=HO2+GLY+NO2
     &   +                 RXRAT(   216 )      ! OLNN+NO=HO2+NO2+ONIT
     &   +    7.4000D-01 * RXRAT(   262 )      ! MO2+MO2=0.7400D+00*HO2+...
     &   +                 RXRAT(   263 )      ! ETHP+MO2=HO2+0.7500D+...
     &   +    8.9400D-01 * RXRAT(   264 )      ! HC3P+MO2=0.8940D+00*HO2+...
     &   +    8.4200D-01 * RXRAT(   265 )      ! HC5P+MO2=0.8420D+00*HO2+...
     &   +                 RXRAT(   266 )      ! ETEP+MO2=HO2+0.1950D+...
     &   +                 RXRAT(   267 )      ! OLTP+MO2=HO2+0.1500D+...
     &   +                 RXRAT(   268 )      ! OLIP+MO2=HO2+0.7500D+...
     &   +    1.3700D+00 * RXRAT(   269 )      ! BENP+MO2=0.1370D+01*HO2+...
     &   +    1.2846D+00 * RXRAT(   270 )      ! TOLP+MO2=0.1285D+01*HO2+...
     &   +    1.3218D+00 * RXRAT(   271 )      ! XYMP+MO2=0.1322D+01*HO2+...
     &   +    1.2846D+00 * RXRAT(   272 )      ! XYEP+MO2=0.1285D+01*HO2+...
     &   +                 RXRAT(   273 )      ! ISOP+MO2=HO2+0.1310D+...
     &   +                 RXRAT(   274 )      ! APIP1+MO2=HO2+0.6800D+...
     &   +                 RXRAT(   275 )      ! APIP2+MO2=HO2+0.7500D+...
     &   +    3.7000D-01 * RXRAT(   276 )      ! APINP1+MO2=0.3700D+00*HO2+...
     &   +    7.5000D-01 * RXRAT(   277 )      ! APINP2+MO2=0.7500D+00*HO2+...
     &   +                 RXRAT(   278 )      ! LIMP1+MO2=HO2+HCHO+0.4200D+...
     &   +                 RXRAT(   279 )      ! LIMP2+MO2=HO2+0.7500D+...
     &   +    3.7000D-01 * RXRAT(   280 )      ! LIMNP1+MO2=0.3700D+00*HO2+...
     &   +    7.5000D-01 * RXRAT(   281 )      ! LIMNP2+MO2=0.7500D+00*HO2+...
     &   +    9.0000D-01 * RXRAT(   282 )      ! ACO3+MO2=0.9000D+00*HO2+...
     &   +    9.0000D-01 * RXRAT(   283 )      ! RCO3+MO2=0.9000D+00*HO2+...
     &   +    5.0000D-01 * RXRAT(   284 )      ! ACTP+MO2=0.5000D+00*HO2+...
     &   +    8.3400D-01 * RXRAT(   285 )      ! MEKP+MO2=0.8340D+00*HO2+...
     &   +                 RXRAT(   286 )      ! KETP+MO2=HO2+0.7500D+...
     &   +    5.0000D-01 * RXRAT(   287 )      ! MACP+MO2=0.5000D+00*HO2+...
     &   +                 RXRAT(   288 )      ! MCP+MO2=HO2+NO2+0.1500D+...
     &   +                 RXRAT(   289 )      ! MVKP+MO2=HO2+0.1160D+...
     &   +                 RXRAT(   290 )      ! UALP+MO2=HO2+0.3050D+00*CO+...
     &   +                 RXRAT(   291 )      ! BALP+MO2=HO2+BAL1+HCHO
     &   +                 RXRAT(   292 )      ! BAL1+MO2=HO2+BAL2+HCHO
     &   +    2.0000D+00 * RXRAT(   293 )      ! ADDC+MO2=0.2000D+01*HO2+...
     &   +                 RXRAT(   294 )      ! MCTP+MO2=HO2+MCTO+HCHO
     &   +                 RXRAT(   295 )      ! ORAP+MO2=HO2+HCHO+GLY
     &   +    2.0000D+00 * RXRAT(   296 )      ! OLNN+MO2=0.2000D+01*HO2+...
     &   +    5.0000D-01 * RXRAT(   297 )      ! OLND+MO2=0.5000D+00*HO2+...
     &   +                 RXRAT(   298 )      ! ADCN+MO2=HO2+0.7000D+...
     &   +                 RXRAT(   299 )      ! XO2+MO2=HO2+HCHO
     &   +    5.0000D-01 * RXRAT(   300 )      ! ETHP+ACO3=0.5000D+00*HO2+...
     &   +    3.9400D-01 * RXRAT(   301 )      ! HC3P+ACO3=0.3940D+00*HO2+...
     &   +    3.4200D-01 * RXRAT(   302 )      ! HC5P+ACO3=0.3420D+00*HO2+...
     &   +    5.0000D-01 * RXRAT(   303 )      ! ETEP+ACO3=0.5000D+00*HO2+...
     &   +    5.0000D-01 * RXRAT(   304 )      ! OLTP+ACO3=0.5000D+00*HO2+...
     &   +    5.0000D-01 * RXRAT(   305 )      ! OLIP+ACO3=0.5000D+00*HO2+...
     &   +                 RXRAT(   306 )      ! BENP+ACO3=HO2+0.7000D+...
     &   +    9.1460D-01 * RXRAT(   307 )      ! TOLP+ACO3=0.9146D+00*HO2+...
     &   +    9.5180D-01 * RXRAT(   308 )      ! XYMP+ACO3=0.9518D+00*HO2+...
     &   +    9.1460D-01 * RXRAT(   309 )      ! XYEP+ACO3=0.9146D+00*HO2+...
     &   +    5.0000D-01 * RXRAT(   310 )      ! ISOP+ACO3=0.5000D+00*HO2+...
     &   +    6.3000D-01 * RXRAT(   311 )      ! APIP1+ACO3=0.6300D+00*HO2+...
     &   +    6.3000D-01 * RXRAT(   315 )      ! LIMP1+ACO3=0.6300D+00*HO2+...
     &   +    3.3000D-01 * RXRAT(   322 )      ! MEKP+ACO3=0.3300D+00*HO2+...
     &   +    5.0000D-01 * RXRAT(   323 )      ! KETP+ACO3=0.5000D+00*HO2+...
     &   +    5.0000D-01 * RXRAT(   325 )      ! MCP+ACO3=0.5000D+00*HO2+...
     &   +    5.0000D-01 * RXRAT(   326 )      ! MVKP+ACO3=0.5000D+00*HO2+...
     &   +    5.0000D-01 * RXRAT(   327 )      ! UALP+ACO3=0.5000D+00*HO2+...
     &   +    2.0000D+00 * RXRAT(   330 )      ! ADDC+ACO3=0.2000D+01*HO2+...
     &   +                 RXRAT(   331 )      ! MCTP+ACO3=HO2+MO2+MCTO
     &   +                 RXRAT(   333 )      ! OLNN+ACO3=HO2+MO2+ONIT
     &   +                 RXRAT(   335 )      ! ADCN+ACO3=HO2+MO2+0.7000D+...
     &   +                 RXRAT(   338 )      ! MO2+NO3=HO2+HCHO+NO2
     &   +                 RXRAT(   339 )      ! ETHP+NO3=HO2+NO2+ACD
     &   +    2.5400D-01 * RXRAT(   340 )      ! HC3P+NO3=0.2540D+00*HO2+...
     &   +    4.8800D-01 * RXRAT(   341 )      ! HC5P+NO3=0.4880D+00*HO2+...
     &   +                 RXRAT(   342 )      ! ETEP+NO3=HO2+NO2+0.1600D+...
     &   +    7.9000D-01 * RXRAT(   343 )      ! OLTP+NO3=0.7900D+00*HO2+...
     &   +    8.6000D-01 * RXRAT(   344 )      ! OLIP+NO3=0.8600D+00*HO2+...
     &   +                 RXRAT(   345 )      ! BENP+NO3=HO2+NO2+0.0000D+...
     &   +    9.1460D-01 * RXRAT(   346 )      ! TOLP+NO3=0.9146D+00*HO2+...
     &   +    9.5180D-01 * RXRAT(   347 )      ! XYMP+NO3=0.9518D+00*HO2+...
     &   +    9.1460D-01 * RXRAT(   348 )      ! XYEP+NO3=0.9146D+00*HO2+...
     &   +                 RXRAT(   349 )      ! ISOP+NO3=HO2+NO2+0.7500D+...
     &   +                 RXRAT(   350 )      ! APIP1+NO3=HO2+NO2+ALD+KET
     &   +                 RXRAT(   351 )      ! LIMP1+NO3=HO2+NO2+0.3850D+...
     &   +    6.7000D-01 * RXRAT(   355 )      ! MEKP+NO3=0.6700D+00*HO2+...
     &   +                 RXRAT(   356 )      ! KETP+NO3=HO2+NO2+DCB1
     &   +                 RXRAT(   358 )      ! MCP+NO3=HO2+NO2+HCHO+HKET
     &   +    3.0000D-01 * RXRAT(   359 )      ! MVKP+NO3=0.3000D+00*HO2+...
     &   +                 RXRAT(   360 )      ! UALP+NO3=HO2+NO2+0.6100D+...
     &   +                 RXRAT(   363 )      ! ADDC+NO3=HO2+NO2+0.3200D+...
     &   +                 RXRAT(   365 )      ! ORAP+NO3=HO2+GLY+NO2
     &   +                 RXRAT(   366 )      ! OLNN+NO3=HO2+NO2+ONIT
     &   +                 RXRAT(   369 )      ! OLNN+OLNN=HO2+0.2000D+01*ONIT
     &   +    5.0000D-01 * RXRAT(   370 )      ! OLNN+OLND=0.5000D+00*HO2+...
     &   +    4.8000D-01 * RXRAT(   375 )      ! APIP2+APIP1=0.4800D+00*HO2+...
     &   +    4.8000D-01 * RXRAT(   376 )      ! APIP2+LIMP1=0.4800D+00*HO2+...
     &   +    4.8000D-01 * RXRAT(   377 )      ! APIP2+ISOP=0.4800D+00*HO2+...
     &   +    4.8000D-01 * RXRAT(   378 )      ! LIMP2+APIP1=0.4800D+00*HO2+...
     &   +    4.8000D-01 * RXRAT(   379 )      ! LIMP2+LIMP1=0.4800D+00*HO2+...
     &   +    4.8000D-01 * RXRAT(   380 )      ! LIMP2+ISOP=0.4800D+00*HO2+...
     &   +    4.8000D-01 * RXRAT(   381 )      ! APINP2+APIP1=0.4800D+...
     &   +    4.8000D-01 * RXRAT(   382 )      ! APINP2+LIMP1=0.4800D+...
     &   +    4.8000D-01 * RXRAT(   383 )      ! APINP2+ISOP=0.4800D+00*HO2+...
     &   +    4.8000D-01 * RXRAT(   384 )      ! LIMNP2+APIP1=0.4800D+...
     &   +    4.8000D-01 * RXRAT(   385 )      ! LIMNP2+LIMP1=0.4800D+...
     &   +    4.8000D-01 * RXRAT(   386 )      ! LIMNP2+ISOP=0.4800D+00*HO2+...
     &   +    5.6000D-01 * RXRAT(   391 )      ! ACRO+O3=0.5600D+00*HO2+...
     &   +    4.7700D-01 * RXRAT(   393 )      ! ACRO=0.4770D+00*HO2+...
     &   +    9.6800D-01 * RXRAT(   395 )      ! BDE13P+NO=0.9680D+00*HO2+...
     &   +                 RXRAT(   396 )      ! BDE13P+NO3=HO2+NO2+0.9250D+...
     &   +    8.7000D-01 * RXRAT(   398 )      ! BDE13P+MO2=0.8700D+00*HO2+...
     &   +    8.0000D-01 * RXRAT(   399 )      ! BDE13P+ACO3=0.8000D+00*HO2+...
     &   +    4.2000D-01 * RXRAT(   400 )      ! BDE13+O3=0.4200D+00*HO2+...
     &   +    7.5000D-01 * RXRAT(   403 )      ! FURANO2+NO=0.7500D+00*HO2+...
     &   +                 RXRAT(   456 )      ! VROCP6ALKP2=HO2+VROCP3OXY2
     &   +                 RXRAT(   457 )      ! VROCP5ALKP2=HO2+VROCP2OXY2
     &   +                 RXRAT(   458 )      ! VROCP4ALKP2=HO2+VROCP1OXY1
     &   +                 RXRAT(   459 )      ! VROCP3ALKP2=HO2+VROCP0OXY2
     &   +                 RXRAT(   460 )      ! VROCP2ALKP2=HO2+VROCN1OXY1
     &   +                 RXRAT(   461 )      ! VROCP1ALKP2=HO2+VROCN2OXY2
     &   +                 RXRAT(   462 )      ! HC10P2=HO2+VROCP4OXY2
     &   +    8.6000D-01 * RXRAT(   463 )      ! VROCP6ALKP2+NO=0.8600D+...
     &   +    8.6000D-01 * RXRAT(   464 )      ! VROCP5ALKP2+NO=0.8600D+...
     &   +    8.6000D-01 * RXRAT(   465 )      ! VROCP4ALKP2+NO=0.8600D+...
     &   +    8.6000D-01 * RXRAT(   466 )      ! VROCP3ALKP2+NO=0.8600D+...
     &   +    8.6000D-01 * RXRAT(   467 )      ! VROCP2ALKP2+NO=0.8600D+...
     &   +    8.6000D-01 * RXRAT(   468 )      ! VROCP1ALKP2+NO=0.8600D+...
     &   +    8.8000D-01 * RXRAT(   469 )      ! HC10P2+NO=0.8800D+00*HO2+...
     &   +                 RXRAT(   470 )      ! VROCP6ALKP2+NO3=HO2+...
     &   +                 RXRAT(   471 )      ! VROCP5ALKP2+NO3=HO2+...
     &   +                 RXRAT(   472 )      ! VROCP4ALKP2+NO3=HO2+...
     &   +                 RXRAT(   473 )      ! VROCP3ALKP2+NO3=HO2+...
     &   +                 RXRAT(   474 )      ! VROCP2ALKP2+NO3=HO2+...
     &   +                 RXRAT(   475 )      ! VROCP1ALKP2+NO3=HO2+...
     &   +                 RXRAT(   476 )      ! HC10P2+NO3=HO2+KET+NO2
     &   +    9.9800D-01 * RXRAT(   486 )      ! VROCP6AROP+NO=0.9980D+...
     &   +    9.4050D-01 * RXRAT(   487 )      ! VROCP6AROP+NO3=0.9405D+...
     &   +    1.3105D+00 * RXRAT(   488 )      ! VROCP6AROP+MO2=0.1310D+...
     &   +    9.4050D-01 * RXRAT(   489 )      ! VROCP6AROP+ACO3=0.9405D+...
     &   +    9.9800D-01 * RXRAT(   492 )      ! VROCP5AROP+NO=0.9980D+...
     &   +    9.4050D-01 * RXRAT(   493 )      ! VROCP5AROP+NO3=0.9405D+...
     &   +    1.3105D+00 * RXRAT(   494 )      ! VROCP5AROP+MO2=0.1310D+...
     &   +    9.4050D-01 * RXRAT(   495 )      ! VROCP5AROP+ACO3=0.9405D+...
     &   +    9.9800D-01 * RXRAT(   498 )      ! NAPHP+NO=0.9980D+00*HO2+...
     &   +    9.4050D-01 * RXRAT(   499 )      ! NAPHP+NO3=0.9405D+00*HO2+...
     &   +    1.3105D+00 * RXRAT(   500 )      ! NAPHP+MO2=0.1310D+01*HO2+...
     &   +    9.4050D-01 * RXRAT(   501 )      ! NAPHP+ACO3=0.9405D+00*HO2+...

      P5 = YC0( HO2 ) + P5 * DTC

c..Production of HONO from OH (r19,4 terms )

      R19_4 =   RKI(    53 ) * YC( NO          ) * DTC      ! OH+NO=HONO

c..Remaining HONO production terms
      P19 =    5.0000D-01 * RXRAT(   423 )      ! NO2=0.5000D+00*HONO+...

      P19 = YC0( HONO ) + P19 * DTC

c..Production of HNO4 from HO2 (r21,5 term )

      R21_5 =   RKI(    70 ) * YCP( NO2         ) * DTC      ! HO2+NO2=HNO4

c..Remaining HNO4 production terms
      P21   =   YC0( HNO4 )

c..HO loss terms not in R5_4 & R19_4
      L4 =                 RKI(    48 ) * YC ( HO2         )   ! HO+HO2=
     &   +                 RKI(    57 ) * YC ( HONO        )   ! HO+HONO=NO2
     &   +                 RKI(    60 ) * YC ( NO2         )   ! HO+NO2=HNO3
     &   +                 RKI(    61 ) * YC ( HNO3        )   ! HO+HNO3=NO3
     &   +                 RKI(    72 ) * YC ( HNO4        )   ! HO+HNO4=NO2
     &   +                 RKI(    75 )                        ! HO=MO2
     &   +                 RKI(    76 ) * YC ( ETH         )   ! HO+ETH=ETHP
     &   +                 RKI(    77 ) * YC ( HC3         )   ! HO+HC3=HC3P+...
     &   +                 RKI(    78 ) * YC ( HC5         )   ! HO+HC5=HC5P+...
     &   +                 RKI(    79 ) * YC ( ETE         )   ! HO+ETE=ETEP
     &   +                 RKI(    80 ) * YC ( OLT         )   ! HO+OLT=OLTP
     &   +                 RKI(    81 ) * YC ( OLI         )   ! HO+OLI=OLIP
     &   +    4.7000D-01 * RKI(    83 ) * YC ( BEN         )   ! HO+BEN=0.5300D+...
     &   +    8.2000D-01 * RKI(    84 ) * YC ( TOL         )   ! HO+TOL=0.1800D+...
     &   +    8.3000D-01 * RKI(    85 ) * YC ( XYM         )   ! HO+XYM=0.1700D+...
     &   +    8.2000D-01 * RKI(    86 ) * YC ( XYE         )   ! HO+XYE=0.1800D+...
     &   +                 RKI(    89 ) * YC ( ISO         )   ! HO+ISO=ISOP+...
     &   +                 RKI(    92 ) * YC ( ISHP        )   ! HO+ISHP=ISOP
     &   +                 RKI(    98 ) * YC ( ISHP        )   ! HO+ISHP=0.6000D+...
     &   +                 RKI(   101 ) * YC ( ISON        )   ! HO+ISON=CO+...
     &   +                 RKI(   103 ) * YC ( API         )   ! HO+API=0.9750D+...
     &   +                 RKI(   104 ) * YC ( LIM         )   ! HO+LIM=0.9450D+...
     &   +                 RKI(   105 ) * YC ( PINAL       )   ! HO+PINAL=0.2300D+...
     &   +                 RKI(   106 ) * YC ( LIMAL       )   ! HO+LIMAL=0.7000D+...
     &   +                 RKI(   108 ) * YC ( ACD         )   ! HO+ACD=ACO3
     &   +                 RKI(   109 ) * YC ( ALD         )   ! HO+ALD=RCO3
     &   +                 RKI(   110 ) * YC ( ACT         )   ! HO+ACT=ACTP
     &   +                 RKI(   111 ) * YC ( MEK         )   ! HO+MEK=MEKP
     &   +                 RKI(   112 ) * YC ( KET         )   ! HO+KET=KETP
     &   +                 RKI(   114 ) * YC ( MACR        )   ! HO+MACR=0.5700D+...
     &   +                 RKI(   115 ) * YC ( MVK         )   ! HO+MVK=MVKP
     &   +                 RKI(   116 ) * YC ( UALD        )   ! HO+UALD=0.3130D+...
     &   +                 RKI(   118 ) * YC ( MGLY        )   ! HO+MGLY=ACO3+CO
     &   +    4.8000D-01 * RKI(   119 ) * YC ( DCB1        )   ! HO+DCB1=0.5200D+...
     &   +    4.8000D-01 * RKI(   120 ) * YC ( DCB2        )   ! HO+DCB2=0.5200D+...
     &   +    4.4000D-01 * RKI(   121 ) * YC ( DCB3        )   ! HO+DCB3=0.5600D+...
     &   +                 RKI(   122 ) * YC ( BALD        )   ! HO+BALD=BALP
     &   +    3.8100D-01 * RKI(   123 ) * YC ( PHEN        )   ! HO+PHEN=0.6190D+...
     &   +    4.1600D-01 * RKI(   124 ) * YC ( CSL         )   ! HO+CSL=0.5840D+...
     &   +                 RKI(   125 ) * YC ( MCT         )   ! HO+MCT=MCTO
     &   +    6.5000D-01 * RKI(   130 ) * YC ( OP1         )   ! HO+OP1=0.3500D+...
     &   +    9.9000D-01 * RKI(   131 ) * YC ( OP2         )   ! HO+OP2=0.1000D-...
     &   +    9.9000D-01 * RKI(   132 ) * YC ( OPB         )   ! HO+OPB=0.1000D-...
     &   +                 RKI(   133 ) * YC ( MAHP        )   ! HO+MAHP=MACP
     &   +                 RKI(   135 ) * YC ( ORA2        )   ! HO+ORA2=0.6400D+...
     &   +    6.5000D-01 * RKI(   136 ) * YC ( PAA         )   ! HO+PAA=0.3500D+...
     &   +                 RKI(   137 ) * YC ( PAN         )   ! HO+PAN=XO2+NO3+HCHO
     &   +                 RKI(   138 ) * YC ( PPN         )   ! HO+PPN=XO2+NO3+HCHO
     &   +                 RKI(   139 ) * YC ( MPAN        )   ! HO+MPAN=NO2+HKET
     &   +                 RKI(   140 ) * YC ( ONIT        )   ! HO+ONIT=HC3P+NO2
     &   +                 RKI(   141 ) * YC ( TRPN        )   ! HO+TRPN=HOM
     &   +                 RKI(   388 ) * YC ( VROCIOXY    )   ! HO+VROCIOXY=...
     &   +                 RKI(   389 ) * YC ( SLOWROC     )   ! HO+SLOWROC=ETHP+...
     &   +                 RKI(   390 ) * YC ( ACRO        )   ! HO+ACRO=0.5700D+...
     &   +    6.6700D-01 * RKI(   394 ) * YC ( BDE13       )   ! HO+BDE13=0.3330D+...
     &   +    5.1000D-01 * RKI(   402 ) * YC ( FURAN       )   ! HO+FURAN=0.4900D+...
     &   +    3.4000D-01 * RKI(   405 ) * YC ( FURANONE    )   ! HO+FURANONE=...
     &   +                 RKI(   414 ) * YC ( SESQ        )   ! HO+SESQ=SESQRO2
     &   +                 RKI(   428 ) * YC ( VROCP6ALK   )   ! HO+VROCP6ALK=...
     &   +                 RKI(   429 ) * YC ( VROCP5ALK   )   ! HO+VROCP5ALK=...
     &   +                 RKI(   430 ) * YC ( VROCP4ALK   )   ! HO+VROCP4ALK=...
     &   +                 RKI(   431 ) * YC ( VROCP3ALK   )   ! HO+VROCP3ALK=...
     &   +                 RKI(   432 ) * YC ( VROCP2ALK   )   ! HO+VROCP2ALK=...
     &   +                 RKI(   433 ) * YC ( VROCP1ALK   )   ! HO+VROCP1ALK=...
     &   +                 RKI(   434 ) * YC ( HC10        )   ! HO+HC10=HC10P
     &   +    8.4000D-01 * RKI(   484 ) * YC ( VROCP6ARO   )   ! HO+VROCP6ARO=...
     &   +    8.4000D-01 * RKI(   490 ) * YC ( VROCP5ARO   )   ! HO+VROCP5ARO=...
     &   +    8.4000D-01 * RKI(   496 ) * YC ( NAPH        )   ! HO+NAPH=0.1600D+...

      L4    = 1.0D0 + L4 * DTC + R5_4 + R19_4

c..HO2 loss terms not included in R4_5 & R21_5 (except for HO2+HO2 )
      L5 =                 RKI(    48 ) * YC ( HO          )   ! HO2+HO=
     &   +                 RKI(    55 ) * YC ( NO          )   ! HO2+NO=HNO3
     &   +    3.0000D-01 * RKI(    63 ) * YC ( NO3         )   ! HO2+NO3=0.7000D+...
     &   +                 RKI(    90 ) * YC ( ISOP        )   ! HO2+ISOP=ISHP
     &   +                 RKI(    95 ) * YC ( IPN         )   ! HO2+IPN=0.8000D+...
     &   +                 RKI(   223 ) * YC ( MO2         )   ! HO2+MO2=OP1
     &   +                 RKI(   224 ) * YC ( ETHP        )   ! HO2+ETHP=OP2
     &   +                 RKI(   225 ) * YC ( HC3P        )   ! HO2+HC3P=OP2
     &   +                 RKI(   226 ) * YC ( HC5P        )   ! HO2+HC5P=OP2
     &   +                 RKI(   227 ) * YC ( ETEP        )   ! HO2+ETEP=OP2
     &   +                 RKI(   228 ) * YC ( OLTP        )   ! HO2+OLTP=OP2
     &   +                 RKI(   229 ) * YC ( OLIP        )   ! HO2+OLIP=OP2
     &   +                 RKI(   230 ) * YC ( BENP        )   ! HO2+BENP=0.6021D+...
     &   +                 RKI(   231 ) * YC ( TOLP        )   ! HO2+TOLP=0.7195D+...
     &   +                 RKI(   232 ) * YC ( XYMP        )   ! HO2+XYMP=0.4820D-...
     &   +                 RKI(   233 ) * YC ( XYEP        )   ! HO2+XYEP=0.8540D-...
     &   +                 RKI(   234 ) * YC ( ISOP        )   ! HO2+ISOP=ISHP
     &   +                 RKI(   235 ) * YC ( APIP1       )   ! HO2+APIP1=OPB
     &   +                 RKI(   236 ) * YC ( APIP2       )   ! HO2+APIP2=HOM
     &   +                 RKI(   237 ) * YC ( APINP1      )   ! HO2+APINP1=TRPN
     &   +                 RKI(   238 ) * YC ( APINP2      )   ! HO2+APINP2=HOM
     &   +                 RKI(   239 ) * YC ( LIMP1       )   ! HO2+LIMP1=OPB
     &   +                 RKI(   240 ) * YC ( LIMP2       )   ! HO2+LIMP2=HOM
     &   +                 RKI(   241 ) * YC ( LIMNP1      )   ! HO2+LIMNP1=TRPN
     &   +                 RKI(   242 ) * YC ( LIMNP2      )   ! HO2+LIMNP2=HOM
     &   +                 RKI(   243 ) * YC ( PINALP      )   ! HO2+PINALP=OPB
     &   +                 RKI(   244 ) * YC ( LIMALP      )   ! HO2+LIMALP=OPB
     &   +    5.6000D-01 * RKI(   245 ) * YC ( ACO3        )   ! HO2+ACO3=0.4400D+...
     &   +    5.6000D-01 * RKI(   246 ) * YC ( RCO3        )   ! HO2+RCO3=0.4400D+...
     &   +    8.5000D-01 * RKI(   247 ) * YC ( ACTP        )   ! HO2+ACTP=0.1500D+...
     &   +                 RKI(   248 ) * YC ( MEKP        )   ! HO2+MEKP=OP2
     &   +                 RKI(   249 ) * YC ( KETP        )   ! HO2+KETP=OP2
     &   +                 RKI(   250 ) * YC ( MACP        )   ! HO2+MACP=MAHP
     &   +                 RKI(   251 ) * YC ( MCP         )   ! HO2+MCP=MAHP
     &   +                 RKI(   252 ) * YC ( MVKP        )   ! HO2+MVKP=OP2
     &   +                 RKI(   253 ) * YC ( UALP        )   ! HO2+UALP=OP2
     &   +                 RKI(   254 ) * YC ( ADDC        )   ! HO2+ADDC=OP2
     &   +                 RKI(   255 ) * YC ( CHO         )   ! HO2+CHO=CSL
     &   +                 RKI(   256 ) * YC ( MCTP        )   ! HO2+MCTP=OP2
     &   +                 RKI(   257 ) * YC ( ORAP        )   ! HO2+ORAP=OP2
     &   +                 RKI(   258 ) * YC ( OLNN        )   ! HO2+OLNN=ONIT
     &   +                 RKI(   259 ) * YC ( OLND        )   ! HO2+OLND=ONIT
     &   +                 RKI(   260 ) * YC ( ADCN        )   ! HO2+ADCN=OP2
     &   +                 RKI(   261 ) * YC ( XO2         )   ! HO2+XO2=OP2
     &   +                 RKI(   397 ) * YC ( BDE13P      )   ! HO2+BDE13P=OP2
     &   +    2.8000D-01 * RKI(   404 ) * YC ( FURANO2     )   ! HO2+FURANO2=...
     &   +                 RKI(   410 ) * YC ( SESQNRO2    )   ! HO2+SESQNRO2=...
     &   +                 RKI(   415 ) * YC ( SESQRO2     )   ! HO2+SESQRO2=...
     &   +                 RKI(   449 ) * YC ( VROCP6ALKP  )   ! HO2+VROCP6ALKP=...
     &   +                 RKI(   450 ) * YC ( VROCP5ALKP  )   ! HO2+VROCP5ALKP=...
     &   +                 RKI(   451 ) * YC ( VROCP4ALKP  )   ! HO2+VROCP4ALKP=...
     &   +                 RKI(   452 ) * YC ( VROCP3ALKP  )   ! HO2+VROCP3ALKP=...
     &   +                 RKI(   453 ) * YC ( VROCP2ALKP  )   ! HO2+VROCP2ALKP=...
     &   +                 RKI(   454 ) * YC ( VROCP1ALKP  )   ! HO2+VROCP1ALKP=...
     &   +                 RKI(   455 ) * YC ( HC10P       )   ! HO2+HC10P=OP2
     &   +                 RKI(   477 ) * YC ( VROCP6ALKP2 )   ! HO2+VROCP6ALKP2=...
     &   +                 RKI(   478 ) * YC ( VROCP5ALKP2 )   ! HO2+VROCP5ALKP2=...
     &   +                 RKI(   479 ) * YC ( VROCP4ALKP2 )   ! HO2+VROCP4ALKP2=...
     &   +                 RKI(   480 ) * YC ( VROCP3ALKP2 )   ! HO2+VROCP3ALKP2=...
     &   +                 RKI(   481 ) * YC ( VROCP2ALKP2 )   ! HO2+VROCP2ALKP2=...
     &   +                 RKI(   482 ) * YC ( VROCP1ALKP2 )   ! HO2+VROCP1ALKP2=...
     &   +                 RKI(   483 ) * YC ( HC10P2      )   ! HO2+HC10P2=...
     &   +                 RKI(   485 ) * YC ( VROCP6AROP  )   ! HO2+VROCP6AROP=...
     &   +                 RKI(   491 ) * YC ( VROCP5AROP  )   ! HO2+VROCP5AROP=...
     &   +                 RKI(   497 ) * YC ( NAPHP       )   ! HO2+NAPHP=...

      L5    = 1.0D0 + L5 * DTC + R4_5 + R21_5

c..HONO loss terms not included in R4_19
      L19 =                 RKI(    57 ) * YC ( HO          )   ! HONO+HO=NO2

      L19   = 1.0D0 + L19 * DTC + R4_19

c..HNO4 loss terms not included in R5_21
      L21 =    2.0000D-01 * RKI(     9 )                        ! HNO4=0.8000D+...
     &    +                 RKI(    72 ) * YC ( HO          )   ! HNO4+HO=NO2

      L21   = 1.0D0 + L21 * DTC + R5_21

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  Solution section
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c..compute terms used to calculate a,b & c
      L21_INV = 1.0D0 / L21
      T1 = 1.0D0 / ( L4 * L19 - R4_19 * R19_4 )
      T2 = R5_4 * T1
      T3 = R5_21 * L21_INV

c..solve quadratic equation for HO2
      A = 2.0D0 * ( RKI(    49 ) + RKI(    50 ) ) * DTC

      B = L5 - T3 * R21_5 - T2 * R4_5 * L19

      C = P5 + T3 * P21 + T2 * ( P4 * L19 + P19 * R4_19 )

      Q = -0.5D0 * ( B + SIGN( 1.0D0, B ) * SQRT( B * B + 4.0D0 * A * C ) )

      YCP( HO2 ) = MAX( Q / A , -C / Q  )

c..compute remaining species concentrations
      YCP( HO ) = ( ( P4 + R4_5 * YCP( HO2 ) ) * L19 + R4_19 * P19 ) * T1

      YCP( HNO4 ) = ( P21 + R21_5 * YCP( HO2 ) ) * L21_INV

      YCP( HONO ) = ( P19 + R19_4 * YCP( HO ) ) / L19

      RETURN

      END


